/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.gui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.pietschy.wizard.models.StaticModel;

import repast.simphony.data.db.outputter.engine.DbOutputterDescriptor;
import repast.simphony.data2.engine.DataSetDescriptor;
import repast.simphony.data2.wizard.DataSetModel;

/**
 * repast.simphony.data.db
 * 
 * @author Sascha Holzhauer
 * @date 13.04.2010
 * 
 */
public class DbOutputterModel extends StaticModel implements DataSetModel<DbOutputterDescriptor> {

	private List<DataSetDescriptor> dataSets;

	private DbOutputterDescriptor descriptor;

	/**
	 * @param path the wizard's path
	 * @param scenario the related scenario
	 * @param contextID the related context's ID
	 */
	public DbOutputterModel(List<DataSetDescriptor> dataSets) {
		this(dataSets, new DbOutputterDescriptor("A Db Outputter"));
			
			
	}

	  public DbOutputterModel(List<DataSetDescriptor> dataSets, DbOutputterDescriptor descriptor) {
		    this.descriptor = descriptor;
		    this.dataSets = new ArrayList<DataSetDescriptor>(dataSets);
		    Collections.sort(this.dataSets, new Comparator<DataSetDescriptor>(){
		      @Override
		      public int compare(DataSetDescriptor arg0, DataSetDescriptor arg1) {
		        return arg0.getName().compareTo(arg1.getName());
		      }
		    });
		  }
	  
	  public String getName() {
		    return descriptor.getName();
		  }
		  
		  public List<DataSetDescriptor> getDataSets() {
		    return dataSets;
		  }

		  public void setName(String actionName) {
		    descriptor.setName(actionName);
		  }

	/**
	 * @return the outputter descriptor
	 */
	public DbOutputterDescriptor getDescriptor() {
		return descriptor;
	}

	/**
	 * @param descriptor  the outputter descriptor
	 */
	public void setDescriptor(DbOutputterDescriptor descriptor) {
		this.descriptor = descriptor;
	}
}