/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.gui;



import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashSet;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.pietschy.wizard.InvalidStateException;

import repast.simphony.data.db.outputter.engine.DbOutputterDescriptor;
import repast.simphony.util.Settings;
import simphony.util.messages.MessageCenter;
import org.jdesktop.swingx.autocomplete.AutoCompleteDecorator;
import org.jdesktop.swingx.autocomplete.AutoCompleteDocument;
import org.jdesktop.swingx.autocomplete.ListAdaptor;
import repast.simphony.freezedry.datasource.DBConnectionInfo;
import repast.simphony.freezedry.datasource.DBUtils;
import repast.simphony.util.wizard.ModelAwarePanelStep;

import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpec;
import com.jgoodies.forms.layout.RowSpec;
import com.jgoodies.forms.layout.Sizes;



/**
 * repast.simphony.data.db
 * 
 * The DB Outputter Wizard Step 2
 * 
 * @author Sascha Holzhauer
 * @date 13.04.2010
 * 
 */
public class DbPropertiesStep extends ModelAwarePanelStep<DbOutputterModel> {

	private static final long			serialVersionUID	= 8420028182447790071L;

	private static final MessageCenter	LOG					= MessageCenter.getMessageCenter(DbPropertiesStep.class);

	class CompleteHandler implements DocumentListener, ItemListener {
		public void insertUpdate(DocumentEvent e) {
			update();
		}

		public void removeUpdate(DocumentEvent e) {
			update();
		}

		public void changedUpdate(DocumentEvent e) {
			update();
		}

		public void itemStateChanged(ItemEvent e) {
			update();
		}

		private void update() {
			setComplete(urlValid() && driverValid() && userInfoComplete());
		}

		private boolean driverValid() {
			try {
				Class.forName(driverField.getText());
				return true;
			} catch (ClassNotFoundException ex) {
				return false;
			}
		}

		private boolean urlValid() {
			return urlField.getText().indexOf(':') != urlField.getText().lastIndexOf(':');
			// url should be like: jdbc:<driver name>:<driver connection string>;
		}

		private boolean userInfoComplete() {
			if (loginBox.isSelected()) {
				if (userNameField.getText().equals("") || passwordField.getPassword().equals("")) {
					return false;
				}
			}
			return true;
		}

	}

	private CompleteHandler		completeHandler;

	private DefaultListModel	urlListModel;

	private DefaultListModel	driverListModel;

	JButton testConnectionButton;
	
	/**
	 * 
	 */
	public DbPropertiesStep() {
		super("Database Connection Information", "Enter the necessary information for connecting to the database");
		completeHandler = new CompleteHandler();
		initComponents();
	}

	private void loginBoxItemStateChanged(ItemEvent e) {
		updateLoginFields();
	}

	private void updateLoginFields() {
		completeHandler.update();
	}

	private void initComponents() {
		DefaultComponentFactory compFactory = DefaultComponentFactory.getInstance();
		separator1 = compFactory.createSeparator("Database Connection Properties");
		label1 = new JLabel();
		urlField = new JTextField();
		label2 = new JLabel();
		driverField = new JTextField();
		separator2 = compFactory.createSeparator("Database User Properties");
		label3 = new JLabel();
		userNameField = new JTextField();
		label4 = new JLabel();
		passwordField = new JPasswordField();
		label8 = new JLabel();
		loginBox = new JCheckBox();
		label5 = new JLabel();
		label6 = new JLabel();
		CellConstraints cc = new CellConstraints();

		testConnectionButton = new JButton("Test DB Connection");
		testConnectionButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				DBConnectionInfo dBInfo = new DBConnectionInfo(userNameField.getText(), new String(passwordField
						.getPassword()), urlField.getText(), driverField.getText());
				if (DBUtils.validateConnectionSettings(dBInfo, "")) {
					// store data back to wizard when connection was successful:
					userNameField.setText(dBInfo.userName);
					passwordField.setText(dBInfo.password);
					urlField.setText(dBInfo.dbURL);
					driverField.setText(dBInfo.driverName);
				}
			}
		});

		// ======== this ========
		setLayout(new FormLayout(new ColumnSpec[] { FormFactory.DEFAULT_COLSPEC,
				FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
				new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW) }, new RowSpec[] {
				FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
				FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
				FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
				FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC}));
		add(separator1, cc.xywh(1, 1, 3, 1));

		// ---- label1 ----
		label1.setText("URL");
		add(label1, cc.xywh(1, 3, 1, 2));
		add(urlField, cc.xy(3, 3));

		// ---- label2 ----
		label2.setText("Driver");
		add(label2, cc.xy(1, 5));
		add(driverField, cc.xy(3, 5));
		add(separator2, cc.xywh(1, 7, 3, 1));

		// ---- label3 ----
		label3.setText("User Name");
		add(label3, cc.xy(1, 9));
		add(userNameField, cc.xy(3, 9));

		// ---- label4 ----
		label4.setText("Password");
		add(label4, cc.xy(1, 11));
		add(passwordField, cc.xy(3, 11));

		// ---- label8 ----
		label8.setText("Store Login Details");
		add(label8, cc.xy(1, 13));

		// ---- loginBox ----
		loginBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				loginBoxItemStateChanged(e);
			}
		});
		add(loginBox, cc.xy(3, 13));

		// ---- label5 ----
		label5.setText("Note - If no user information is specified, you will be prompted when it is needed.");
		add(label5, cc.xywh(1, 15, 3, 1));

		// ---- label6 ----
		label6.setText("Warning - Username and Passwords are saved in clear text and therefore not secure.");
		label6.setForeground(Color.red);
		add(label6, cc.xywh(1, 19, 3, 1));
		// JFormDesigner - End of component initialization //GEN-END:initComponents

		add(testConnectionButton, cc.xywh(1, 22, 3, 1));

		updateLoginFields();

		setupAutoComplete();

		urlField.getDocument().addDocumentListener(completeHandler);
		passwordField.getDocument().addDocumentListener(completeHandler);
		driverField.getDocument().addDocumentListener(completeHandler);
	}

	// JFormDesigner - Variables declaration - DO NOT MODIFY //GEN-BEGIN:variables
	// Generated using JFormDesigner non-commercial license
	private JComponent			separator1;
	private JLabel				label1;

	/**
	 * DB URL
	 */
	protected JTextField		urlField;
	private JLabel				label2;

	/**
	 * driver class
	 */
	protected JTextField		driverField;
	private JComponent			separator2;
	private JLabel				label3;

	/**
	 * user-name field
	 */
	protected JTextField		userNameField;
	private JLabel				label4;

	/**
	 * password field
	 */
	protected JPasswordField	passwordField;
	private JLabel				label8;

	/**
	 * True if DB login info (user-name, password shall be stored
	 */
	protected JCheckBox			loginBox;
	private JLabel				label5;
	private JLabel				label6;

	/**
	 * 
	 */
	protected void setupAutoComplete() {
		JList urlList = new JList();
		urlListModel = new DefaultListModel();
		urlList.setModel(urlListModel);
		ListAdaptor adaptor = new ListAdaptor(urlList, urlField);
		AutoCompleteDocument document = new AutoCompleteDocument(adaptor, false);
		AutoCompleteDecorator.decorate(urlField, document, adaptor);

		try {
			@SuppressWarnings("unchecked")
			HashSet<Object> urlSet = (HashSet<Object>) Settings.getRegistry().get("JDBCDataStep.URLS");
			if (urlSet != null) {
				for (Object url : urlSet) {
					urlListModel.addElement(url.toString());
				}
			}
		} catch (Exception ex) {
			LOG.info("Error loading urls from registry", ex);
		}

		JList driverList = new JList();
		driverListModel = new DefaultListModel();
		driverList.setModel(driverListModel);
		adaptor = new ListAdaptor(driverList, driverField);
		document = new AutoCompleteDocument(adaptor, false);
		AutoCompleteDecorator.decorate(driverField, document, adaptor);

		try {
			@SuppressWarnings("unchecked")
			HashSet<Object> driverSet = (HashSet<Object>) Settings.getRegistry().get("JDBCDataStep.DRIVERS");
			if (driverSet != null) {
				for (Object driver : driverSet) {
					driverListModel.addElement(driver.toString());
				}
			}
		} catch (Exception ex) {
			LOG.info("Error loading drivers from registry", ex);
		}
	}

	/**
	 * @param string the string to test
	 * @return true if the string was not null and not empty
	 */
	protected boolean notNull(String string) {
		return string != null && !string.equals("");
	}

	/**
	 * @see org.pietschy.wizard.PanelWizardStep#prepare()
	 */
	@Override
	public void prepare() {
		super.prepare();

		if (model != null && model.getDescriptor() != null) {
			DbOutputterDescriptor loader = model.getDescriptor();

			if (notNull(loader.getDriverName())) {
				driverField.setText(loader.getDriverName());
			}

			if (notNull(loader.getDbURL())) {
				urlField.setText(loader.getDbURL());
			}

			if (notNull(loader.getUsername())) {
				userNameField.setText(loader.getUsername());
			}

			if (notNull(loader.getPassword())) {
				passwordField.setText(loader.getPassword());
			}

			loginBox.setSelected(loader.isStoringLoginInfo());
			loginBox.addItemListener(new ItemListener() {

				@Override
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.DESELECTED) {
						userNameField.setText("");
						userNameField.setEnabled(false);
						passwordField.setText("");
						passwordField.setEnabled(false);
						testConnectionButton.setEnabled(false);
					}
					if (e.getStateChange() == ItemEvent.SELECTED) {
						userNameField.setEnabled(true);
						passwordField.setEnabled(true);
						testConnectionButton.setEnabled(true);
					}
				}
			});
		}
	}

	/**
	 * Store the user entries into the outputter descriptor. l
	 * 
	 * @see repast.simphony.dataLoader.wizard.JDBCDataStep#applyState()
	 */
	@Override
	public void applyState() throws InvalidStateException {
		super.applyState();
		DbOutputterDescriptor builder = model.getDescriptor();
		builder.setDriverName(driverField.getText());
		builder.setDbURL(urlField.getText());
		builder.setUsername(userNameField.getText());
		builder.setPassword(new String(passwordField.getPassword()));
		builder.setStoringLoginInfo(loginBox.isSelected());
	}
}
